/*=============================================================================
	Class cache definitions
	Move to XC_Core later, do not link here
=============================================================================*/

#ifndef _INC_XC_CLASSCACHE
#define _INC_XC_CLASSCACHE

//A standard PlayerPawn iterator list
struct FIteratorPList
{
	APlayerPawn* P;
	FIteratorPList* Next;
	FIteratorPList() {};
	FIteratorPList( APlayerPawn* nP, FIteratorPList* InNext)
	:	P(nP)
	,	Next(InNext)
	{};
};

//A standard cached property
struct FPropertyCache
{
	FPropertyCache* Next;
	UProperty* Property;
	FPropertyCache() {};
	FPropertyCache( UProperty* InProperty, FPropertyCache* InNext)
	:	Next( InNext)
	,	Property( InProperty)
	{};
};

//Base type of Class cache, subdefine for different property caching types
//The first object to be added should be the highest intended class
struct XC_ENGINE_API FClassPropertyCache
{
	FClassPropertyCache* Parent;
	FClassPropertyCache* Next;
	UClass* Class;
	UBOOL bProps;
	FPropertyCache* Properties;

	FClassPropertyCache* GetCache( UClass* Other)
	{
		for ( FClassPropertyCache* Cached=this ; Cached ; Cached=Cached->Next )
			if ( Cached->Class == Other )
				return Cached;
		return NULL;
	}
	UBOOL IsCached( UClass* Other)
	{
		return GetCache( Other) != NULL;
	}

	FClassPropertyCache( UClass* MasterClass=UObject::StaticClass() )
	:	Parent( NULL), Next( NULL), Class( MasterClass ), bProps(0), Properties( NULL)
	{};

	FClassPropertyCache( FClassPropertyCache* InNext, UClass* InClass)
	:	Parent( NULL), Next( InNext), Class( InClass), bProps(0), Properties( NULL)
	{
		for ( FClassPropertyCache* FPC=Next ; FPC ; FPC=FPC->Next )
			if ( FPC->Class == Class->GetSuperClass() )
			{
				Parent = FPC;
				break;
			}
	};
	
	void GrabProperties( FMemStack& Mem)
	{
		UProperty* Prop = Class->PropertyLink;
		INT MinOffset = Class->GetSuperClass() ? Class->GetSuperClass()->GetPropertiesSize() : 0;
		while ( Prop && Prop->Offset >= MinOffset ) //Find Travel properties
		{
			if ( AcceptProperty(Prop) )
				Properties = new(Mem) FPropertyCache( Prop, Properties);
			Prop = Prop->PropertyLinkNext;
		}
		bProps = 1;
		if ( Next && !Parent && Class->GetSuperClass() ) //Not last, but needs parent
		{
			Next = CreateParent( Mem);
			Parent = Next;
			Parent->GrabProperties( Mem);
		}
	}

	virtual UBOOL AcceptProperty( UProperty* Property)
	{
		return true;
	}
	virtual FClassPropertyCache* CreateParent( FMemStack& Mem)
	{
		return new(Mem) FClassPropertyCache( Next, Class->GetSuperClass() );
	}
};

#endif
